@@ -12,10 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import sys
+
+import six
+
 from keystone import auth
 from keystone.common import dependency
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 from keystone.openstack.common import log
 
 METHOD_NAME = 'password'
@@ -37,16 +41,24 @@ class UserAuthInfo(object):
         self.user_ref = None
 
     def _assert_domain_is_enabled(self, domain_ref):
-        if not domain_ref.get('enabled'):
-            msg = _('Domain is disabled: %s') % (domain_ref['id'])
-            LOG.warning(msg)
-            raise exception.Unauthorized(msg)
+        try:
+            self.assignment_api.assert_domain_enabled(
+                domain_id=domain_ref['id'],
+                domain=domain_ref)
+        except AssertionError as e:
+            LOG.warning(e)
+            six.reraise(exception.Unauthorized, exception.Unauthorized(e),
+                        sys.exc_info()[2])
 
     def _assert_user_is_enabled(self, user_ref):
-        if not user_ref.get('enabled', True):
-            msg = _('User is disabled: %s') % (user_ref['id'])
-            LOG.warning(msg)
-            raise exception.Unauthorized(msg)
+        try:
+            self.identity_api.assert_user_enabled(
+                user_id=user_ref['id'],
+                user=user_ref)
+        except AssertionError as e:
+            LOG.warning(e)
+            six.reraise(exception.Unauthorized, exception.Unauthorized(e),
+                        sys.exc_info()[2])
 
     def _lookup_domain(self, domain_info):
         domain_id = domain_info.get('id')
@@ -106,7 +118,7 @@ class Password(auth.AuthMethodHandler):
 
     method = METHOD_NAME
 
-    def authenticate(self, context, auth_payload, user_context):
+    def authenticate(self, context, auth_payload, auth_context):
         """Try to authenticate against the identity backend."""
         user_info = UserAuthInfo.create(auth_payload)
 
@@ -116,12 +128,10 @@ class Password(auth.AuthMethodHandler):
             self.identity_api.authenticate(
                 context,
                 user_id=user_info.user_id,
-                password=user_info.password,
-                domain_scope=user_info.domain_id)
+                password=user_info.password)
         except AssertionError:
             # authentication failed because of invalid username or password
             msg = _('Invalid username or password')
             raise exception.Unauthorized(msg)
 
-        if 'user_id' not in user_context:
-            user_context['user_id'] = user_info.user_id
+        auth_context['user_id'] = user_info.user_id
