@@ -16,11 +16,11 @@ import uuid
 from testtools import matchers
 
 from keystone import exception
-from keystone.tests import unit
+from keystone import tests
 from keystone.token.providers import common
 
 
-class TestTokenDataHelper(unit.TestCase):
+class TestTokenDataHelper(tests.TestCase):
     def setUp(self):
         super(TestTokenDataHelper, self).setUp()
         self.load_backends()
@@ -28,8 +28,7 @@ class TestTokenDataHelper(unit.TestCase):
 
     def test_v3_token_data_helper_populate_audit_info_string(self):
         token_data = {}
-        audit_info_bytes = base64.urlsafe_b64encode(uuid.uuid4().bytes)[:-2]
-        audit_info = audit_info_bytes.decode('utf-8')
+        audit_info = base64.urlsafe_b64encode(uuid.uuid4().bytes)[:-2]
         self.v3_data_helper._populate_audit_info(token_data, audit_info)
         self.assertIn(audit_info, token_data['audit_ids'])
         self.assertThat(token_data['audit_ids'], matchers.HasLength(2))
